;
; MPFIT_PARAB
;
; Example of meteor impact
;   * meteorite fragments are found at positions XM, YM
;   * goal is to find location of meteor explosion
;
; Calls MPFIT_CENT as the user function
;
!p.background = 'ffffff'xl
!p.color = '000000'xl

;; Centroid of meteor explosion (God's truth) [km]
xc0 = 5.2d
yc0 = 7.3d
;; Location of observing station [km]
xs0 = -1.2d
ys0 =  0.2d
;; Separation distance is 9.56 km.
radius = 9.56d
;; Sighting angle is 47.97 deg from east
th_sight = 47.97d

if n_elements(xm) EQ 0 then begin
   n = 10
   noise = 1.0d
   ;; Locations of meteorite fragments
   xm = randomn(seed,n)*noise + xc0
   ym = randomn(seed,n)*noise + yc0
endif

; =====================================
print, 'Unconstrained fit...'
plot, xm, ym, psym=1, $
      xrange=[-10,10], yrange=[-10,10], $
      xtitle='X Position', ytitle='Y Position', $
      title='Meteorite Fragment Locations', $
      charsize=1.7
oplot, [xs0], [ys0], psym=6, symsize=3
oplot, !x.crange, [0,0], linestyle=1
oplot, [0,0], !y.crange, linestyle=1

print, 'enter .CONTINUE to overplot best fit...'
stop

p = mpfit('MPFIT_CENT', [1,1d], perror=dp, $
          functargs={X_FRAGMENT: XM, Y_FRAGMENT: YM})
oplot, [p[0]], [p[1]], psym=4, symsize=3, thick=3

print, 'enter .CONTINUE to perform circle-constrained analysis...'
stop

; =====================================
print, 'Constrained by circle...'
plot, xm, ym, psym=1, $
      xrange=[-10,10], yrange=[-10,10], $
      xtitle='X Position', ytitle='Y Position', $
      title='Meteorite Fragment Locations', $
      charsize=1.7
oplot, [xs0], [ys0], psym=6, symsize=3
oplot, !x.crange, [0,0], linestyle=1
oplot, [0,0], !y.crange, linestyle=1
th = dindgen(361)*!dtor
rx = radius*cos(th)+xs0 & ry = radius*sin(th)+ys0
oplot, rx, ry, linestyle=2

print, 'enter .CONTINUE to overplot best fit...'
stop

p = mpfit('MPFIT_CENT', [1,1d], perror=dp, $
          functargs={X_FRAGMENT: XM, Y_FRAGMENT: YM, $
                     X_STATION: XS0, Y_STATION: YS0, RADIUS: RADIUS})
oplot, [p[0]], [p[1]], psym=4, symsize=3, thick=3

print, 'enter .CONTINUE to perform circle-constrained analysis...'
stop

; =====================================
print, 'Constrained by circle and sighting line...'
plot, xm, ym, psym=1, $
      xrange=[-10,10], yrange=[-10,10], $
      xtitle='X Position', ytitle='Y Position', $
      title='Meteorite Fragment Locations', $
      charsize=1.7
oplot, [xs0], [ys0], psym=6, symsize=3
oplot, !x.crange, [0,0], linestyle=1
oplot, [0,0], !y.crange, linestyle=1
th = dindgen(361)*!dtor
rx = radius*cos(th)+xs0 & ry = radius*sin(th)+ys0
oplot, rx, ry, linestyle=2
oplot, !x.crange, (!x.crange-xs0)*tan(th_sight*!dtor)+ys0, linestyle=2

print, 'enter .CONTINUE to overplot best fit...'
stop

p = mpfit('MPFIT_CENT', [1,1d], perror=dp, $
          functargs={X_FRAGMENT: XM, Y_FRAGMENT: YM, $
                     X_STATION: XS0, Y_STATION: YS0, RADIUS: RADIUS, $
                     SIGHT_ANGLE: TH_SIGHT})
oplot, [p[0]], [p[1]], psym=4, symsize=3, thick=3

print, 'enter .CONTINUE to continue with equations only, no data...'
stop

; =====================================
print, 'Equation solver only, no data...'
plot, xm, ym, psym=1, /nodata, $
      xrange=[-10,10], yrange=[-10,10], $
      xtitle='X Position', ytitle='Y Position', $
      title='Meteorite Fragment Locations', $
      charsize=1.7
oplot, [xs0], [ys0], psym=6, symsize=3
oplot, !x.crange, [0,0], linestyle=1
oplot, [0,0], !y.crange, linestyle=1
th = dindgen(361)*!dtor
rx = radius*cos(th)+xs0 & ry = radius*sin(th)+ys0
oplot, rx, ry, linestyle=2
oplot, !x.crange, (!x.crange-xs0)*tan(th_sight*!dtor)+ys0, linestyle=2

print, 'enter .CONTINUE to overplot best fit...'
stop

p = mpfit('MPFIT_CENT', [1,1d], perror=dp, $
          functargs={X_STATION: XS0, Y_STATION: YS0, RADIUS: RADIUS, $
                     SIGHT_ANGLE: TH_SIGHT})
oplot, [p[0]], [p[1]], psym=4, symsize=3, thick=3

print, 'enter .CONTINUE to continue...'
stop

end

